if (!("finalizeConstruction" in ViewPU.prototype)) {
    Reflect.set(ViewPU.prototype, "finalizeConstruction", () => { });
}
interface WebView_Params {
    isGranted?: boolean;
    webviewController?: webview.WebviewController;
    _urlString?: string;
}
import webview from "@ohos:web.webview";
import promptAction from "@ohos:promptAction";
import router from "@ohos:router";
import uri from "@ohos:uri";
import abilityAccessCtrl from "@ohos:abilityAccessCtrl";
import type common from "@ohos:app.ability.common";
import type OpenLinkOptions from "@ohos:app.ability.OpenLinkOptions";
import type { Permissions } from "@ohos:abilityAccessCtrl";
import type Want from "@ohos:app.ability.Want";
import type { BusinessError } from "@ohos:base";
import { PermissionsUtils } from "@bundle:com.example.hongmengdemo/entry/ets/pages/PermissionsUtils";
import camera from "@ohos:multimedia.camera";
import picker from "@ohos:file.picker";
import cameraPicker from "@ohos:multimedia.cameraPicker";
//let context = getContext(this) as common.Context
let context: common.UIAbilityContext = getContext(this) as common.UIAbilityContext; // UIAbilityContext
// context为需要停止的UIAbility实例的AbilityContext
const TAG = '[WebView]';
let mContext = getContext(this) as common.Context;
async function picCamera(video: boolean): Promise<cameraPicker.PickerResult> {
    return new Promise((resolve, reject) => {
        try {
            let pickerProfile: cameraPicker.PickerProfile = {
                cameraPosition: camera.CameraPosition.CAMERA_POSITION_FRONT
            };
            let type = video ? cameraPicker.PickerMediaType.VIDEO : cameraPicker.PickerMediaType.PHOTO;
            let pickerResult = cameraPicker.pick(mContext, [type], pickerProfile);
            console.log("the pick pickerResult is:" + JSON.stringify(pickerResult));
            resolve(pickerResult);
        }
        catch (error) {
            let err = error as BusinessError;
            console.error(`the pick call failed. error code: ${err.code}`);
            reject(error);
        }
    });
}
class WebView extends ViewPU {
    constructor(parent, params, __localStorage, elmtId = -1, paramsLambda = undefined, extraInfo) {
        super(parent, __localStorage, elmtId, extraInfo);
        if (typeof paramsLambda === "function") {
            this.paramsGenerator_ = paramsLambda;
        }
        this.__isGranted = new ObservedPropertySimplePU(false, this, "isGranted");
        this.webviewController = new webview.WebviewController();
        this._urlString = (router.getParams() as routerParams).url;
        this.setInitiallyProvidedValue(params);
        this.finalizeConstruction();
    }
    setInitiallyProvidedValue(params: WebView_Params) {
        if (params.isGranted !== undefined) {
            this.isGranted = params.isGranted;
        }
        if (params.webviewController !== undefined) {
            this.webviewController = params.webviewController;
        }
        if (params._urlString !== undefined) {
            this._urlString = params._urlString;
        }
    }
    updateStateVars(params: WebView_Params) {
    }
    purgeVariableDependenciesOnElmtId(rmElmtId) {
        this.__isGranted.purgeDependencyOnElmtId(rmElmtId);
    }
    aboutToBeDeleted() {
        this.__isGranted.aboutToBeDeleted();
        SubscriberManager.Get().delete(this.id__());
        this.aboutToBeDeletedInternal();
    }
    private __isGranted: ObservedPropertySimplePU<boolean>;
    get isGranted() {
        return this.__isGranted.get();
    }
    set isGranted(newValue: boolean) {
        this.__isGranted.set(newValue);
    }
    private webviewController: webview.WebviewController;
    // 当前加载的链接
    private _urlString: string;
    initialRender() {
        this.observeComponentCreation2((elmtId, isInitialRender) => {
            Column.create();
            Column.height('100%');
            Column.width('100%');
        }, Column);
        this.observeComponentCreation2((elmtId, isInitialRender) => {
            // 资源文件route.html存放路径src/main/resources/rawfile
            Web.create({ src: this._urlString, controller: this.webviewController });
            // 资源文件route.html存放路径src/main/resources/rawfile
            Web.domStorageAccess(true);
            // 资源文件route.html存放路径src/main/resources/rawfile
            Web.javaScriptAccess(true);
            // 资源文件route.html存放路径src/main/resources/rawfile
            Web.fileAccess(true);
            // 资源文件route.html存放路径src/main/resources/rawfile
            Web.onPermissionRequest((event) => {
                if (event) {
                    //TYPE_VIDEO_CAPTURE
                    this.checkPermission().then(() => {
                        event.request.grant(event.request.getAccessibleResource()); // 2. 授权h5页面
                    }).catch((error: BusinessError) => {
                        console.error(TAG, "申请权限异常" + error.message);
                        event.request.deny(); //2.告诉h5页面没有权限
                    });
                }
            });
            // 资源文件route.html存放路径src/main/resources/rawfile
            Web.onShowFileSelector((event) => {
                //.ogm,.ogv,.mpg,.mp4,.mpeg,.m4v,.webm
                console.log('pj', "===>" + event.fileSelector.getAcceptType());
                if (event) {
                    if (event.fileSelector.isCapture()) {
                        //需要录制视频
                        if (event.fileSelector.getAcceptType().includes('.mp4')) {
                            console.log("pj", "========录视频");
                            let result = picCamera(true).then(result => {
                                let str: string[] = [result.resultUri];
                                event.result.handleFileList(str); // 2. 把录制的视频传给h5页面
                            }).catch((error: BusinessError) => {
                                console.error(TAG, "打开相机异常" + error.message);
                            });
                        }
                        else { //拍照
                            console.log("pj", "=======拍照");
                            let result = picCamera(false).then(result => {
                                let str: string[] = [result.resultUri];
                                event.result.handleFileList(str); // 2. 把录制的视频传给h5页面
                            }).catch((error: BusinessError) => {
                                console.error(TAG, "打开相机异常" + error.message);
                            });
                        }
                    }
                    else {
                        // 文件选择器
                        console.info('MyFileUploader onShowFileSelector invoked.');
                        const documentSelectOptions = new picker.DocumentSelectOptions();
                        let uri: string | null = null;
                        const documentViewPicker = new picker.DocumentViewPicker();
                        documentViewPicker.select(documentSelectOptions).then((documentSelectResult) => {
                            uri = documentSelectResult[0];
                            if (event) {
                                event.result.handleFileList([uri]);
                            }
                        }).catch((err: BusinessError) => {
                            console.error(`Invoke documentViewPicker.select failed, code is ${err.code}, message is ${err.message}`);
                        });
                    }
                }
                return true;
            });
            // 资源文件route.html存放路径src/main/resources/rawfile
            Web.onLoadIntercept((event) => {
                console.log("pj", "========event拦截事件=============>" + event);
                if (event) {
                    let url: string = event.data.getRequestUrl();
                    if (url == null) {
                        return;
                    }
                    const uriInstance = new uri.URI(url);
                    console.log("pj", "===========协议是：" + uriInstance.scheme + " url:" + url);
                    if (uriInstance.scheme == 'https' || uriInstance.scheme == 'http') {
                        //return false不处理默认加载
                        return false;
                    }
                    else if (uriInstance.scheme == 'alipays') {
                        //alipays://platformapi/startapp?appId=20000067&url=https%3A%2F%2Fsmlh5.esign.cn%2Fwebserver-identity%2Ffootstone-identity%2Fv2%2Ffaceauth%2Fscan%3FfaceAuthMode%3D2%26flowId%3D3608979457115164745%26timestamp%3D1729847947560
                        // let context = getContext(this) as common.UIAbilityContext;
                        let link: string = url;
                        let openLinkOptions: OpenLinkOptions = {
                            appLinkingOnly: false
                        };
                        context.openLink(link, openLinkOptions).then(() => {
                            //TODO 跳转成功的回调
                        }).catch((err: BusinessError) => {
                            //TODO 跳转识别的回调
                        });
                        return true;
                    }
                    else if (url.startsWith("esign://demo/realBack")) {
                        //esign://demo/realBack?status=true&authFlowId=OF-2ecd9c5e3408000d&lang=zh-CN
                        //实名结果
                        let status = uriInstance.getBooleanQueryValue("status", false);
                        if (status) {
                            //认证成功返回
                            promptAction.showToast({
                                message: '认证成功',
                                duration: 2000
                            });
                            router.pushUrl({
                                url: 'pages/Index'
                            });
                            // context.terminateSelf() 直接结束
                        }
                        return true;
                    }
                    else if (url.startsWith("esign://demo/signBack")) {
                        //TODO 签署schemal
                        // esign://demo/signBack?tsignType=SIGN&tsignCode=0&tsignDes=%E7%AD%BE%E7%BD%B2%E6%88%90%E5%8A%9F  签署结果
                        if (url.includes("signResult")) {
                            let signResult = uriInstance.getBooleanQueryValue("signResult", false);
                            promptAction.showToast({
                                message: '签署结果' + signResult,
                                duration: 2000
                            });
                        }
                        else if (url.includes("tsignCode")) {
                            let tsignCode = uriInstance.getQueryValue("tsignCode");
                            if (tsignCode == "0") {
                                promptAction.showToast({
                                    message: '签署成功',
                                    duration: 2000
                                });
                            }
                            else {
                                promptAction.showToast({
                                    message: '签署失败',
                                    duration: 2000
                                });
                            }
                        }
                        router.pushUrl({
                            url: 'pages/Index'
                        });
                        return true;
                    }
                    else {
                        return false;
                    }
                }
                return false;
            });
        }, Web);
        Column.pop();
    }
    // 相机权限检测
    async checkPermission() {
        // 检查权限
        const permissions: Array<Permissions> = ['ohos.permission.CAMERA'];
        let grantStatusCamera = await PermissionsUtils.checkAccessToken(permissions[0]);
        console.debug(TAG, `checkPermission grantStatus ${grantStatusCamera}`);
        if (grantStatusCamera != abilityAccessCtrl.GrantStatus.PERMISSION_GRANTED) {
            console.debug(TAG, `requestCameraPermission`);
            // 没有授权了
            await this.requestPermission(permissions);
        }
        else {
            this.isGranted = true;
        }
    }
    // 获取权限
    async requestPermission(permissionList: Array<Permissions>) {
        let context = getContext(this) as common.UIAbilityContext;
        let grantStatus = await PermissionsUtils.reqPermissionsFromUser(context, permissionList);
        let length: number = grantStatus.length;
        for (let i = 0; i < length; i++) {
            if (grantStatus[i] === 0) {
                console.info(TAG, `requestPermissionsFromUser success`);
            }
            else {
                console.error(TAG, `requestPermissionsFromUser failed`);
                AlertDialog.show({
                    title: '权限申请',
                    message: '请授权申请相机权限',
                    primaryButton: {
                        value: '取消',
                        action: () => {
                        }
                    },
                    secondaryButton: {
                        value: '确认',
                        action: () => {
                            this.openPermissionsInSystemSettings();
                        }
                    },
                    onWillDismiss: (dismissDialogAction: DismissDialogAction) => {
                        console.debug(TAG, `dialog onWillDismiss reason= + ${JSON.stringify(dismissDialogAction.reason)}`);
                        //不管是点击三键back、左滑/右滑、键盘ESC，还是点击遮障层,或者是下拉关闭都不退出；只有点击关闭按钮时才退出
                        if (dismissDialogAction.reason == DismissReason.CLOSE_BUTTON) {
                            dismissDialogAction.dismiss();
                        }
                    },
                    autoCancel: false,
                    alignment: DialogAlignment.Center
                });
                break;
            }
        }
    }
    // 打开系统权限设置页
    async openPermissionsInSystemSettings(): Promise<void> {
        console.error(TAG, `openPermissionsInSystemSettings`);
        let context = getContext(this) as common.UIAbilityContext;
        let currentBundleName = await PermissionsUtils.getBundleName();
        let wantInfo: Want = {
            bundleName: 'com.huawei.hmos.settings',
            abilityName: 'com.huawei.hmos.settings.MainAbility',
            uri: 'application_info_entry',
            parameters: {
                settingsParamBundleName: currentBundleName // Open the details page of a specified application.
            }
        };
        try {
            context.startAbilityForResult(wantInfo).then((result: common.AbilityResult) => {
                console.info(TAG, `startAbility Success:${result.resultCode}`);
            }).catch((err: BusinessError) => {
                // 处理业务逻辑错误
                console.error(TAG, `startAbilityForResult failed, code is ${err.code}, message is ${err.message}`);
            });
        }
        catch (err) {
            // 处理入参错误异常
            let code = (err as BusinessError).code;
            let message = (err as BusinessError).message;
            console.error(TAG, `startAbilityForResult failed, code is ${code}, message is ${message}`);
        }
    }
    rerender() {
        this.updateDirtyElements();
    }
    static getEntryName(): string {
        return "WebView";
    }
}
class routerParams {
    'url': string;
    'view_file': boolean;
}
registerNamedRoute(() => new WebView(undefined, {}), "", { bundleName: "com.example.hongmengdemo", moduleName: "entry", pagePath: "pages/WebView", pageFullPath: "entry/src/main/ets/pages/WebView", integratedHsp: "false" });
