//
//  SecondViewController.m
//  FaceAuthDemo
//
//  Created by tsign on 2019/3/25.
//  Copyright © 2019 tsign. All rights reserved.
//

#import "SecondViewController.h"
#import "RealnameHelper.h"
#import "UitilClass.h"
#import <WebKit/WebKit.h>

//获取导航栏+状态栏的高度
#define getRectNavAndStatusHight \
({\
CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];\
CGRect rectNav = self.navigationController.navigationBar.frame;\
( rectStatus.size.height+ rectNav.size.height);\
})\



@interface SecondViewController () <WKNavigationDelegate, RealnameDelegate>

@property (strong, nonatomic) WKWebView *wkWebView;
@property (strong, nonatomic) UIProgressView *progressView;
@property (strong, nonatomic) WKWebViewConfiguration *configuration;
@property (strong, nonatomic) WKUserContentController *userContentController;

@end

@implementation SecondViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    self.wkWebView = [[WKWebView alloc]initWithFrame:[UIScreen mainScreen].bounds configuration:self.configuration];
    self.wkWebView.navigationDelegate = self;
    [self.view addSubview:_wkWebView];
    
    [self reloadUrl:self.realnameUrl];
    
    
    
}

//在页面即将展示的时候添加监听
- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [self.wkWebView addObserver:self forKeyPath:@"estimatedProgress" options:NSKeyValueObservingOptionNew context:nil];
}

//在页面消失之前释放
- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [self.wkWebView removeObserver:self forKeyPath:@"estimatedProgress"];
}

- (void)reloadUrl:(NSString *)url {
   
    [self.wkWebView loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:url]]];
    
}

//监听处理方法
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary<NSString *,id> *)change context:(void *)context {
    if ([keyPath isEqualToString:@"estimatedProgress"]) {
        self.progressView.progress = self.wkWebView.estimatedProgress;
        if (self.progressView.progress == 1) {
            /*
             *添加一个简单的动画，将progressView的Height变为1.4倍，在开始加载网页的代理中会恢复为1.5倍
             *动画时长0.25s，延时0.3s后开始动画
             *动画结束后将progressView隐藏
             */
            __weak typeof (self)weakSelf = self;
            [UIView animateWithDuration:0.25f delay:0.3f options:UIViewAnimationOptionCurveEaseOut animations:^{
                weakSelf.progressView.transform = CGAffineTransformMakeScale(1.0f, 1.4f);
            } completion:^(BOOL finished) {
                weakSelf.progressView.hidden = YES;
                
            }];
        }
    }else{
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}

- (void)closeCtrl {
    [self.navigationController popViewControllerAnimated:YES];
}

- (BOOL)handleRequest:(NSURLRequest *)request {
    URL_HANDLE_RESULT handleResult = [RealnameHelper handleRealnameURL:request.URL delegate:self];
    return handleResult != URL_HANDLE_RESULT_CANCEL;
}

#pragma mark - RealnameDelegate
- (void)realnameJumptoAlipay {
}

- (void)realnameZhimaCallback:(NSString *)callbackUrl {
    [self reloadUrl:callbackUrl];
}

- (void)realnameResult:(REALNAME_RESULT)result {
    if (result == REALNAME_RESULT_SUCCESS) {
        [UitilClass creatAlertControllerAlert:@"实名成功" target:self handle:^{
            [self closeCtrl];
        }];
    }else if(result == REALNAME_RESULT_FAILED){
        [UitilClass creatAlertControllerAlert:@"实名失败" target:self handle:nil];
    }
}

- (void)signResult:(BOOL)result {
    if (result) {
        [UitilClass creatAlertControllerAlert:@"签署成功" target:self handle:^{
            [self closeCtrl];
        }];
    } else {
        [UitilClass creatAlertControllerAlert:@"签署失败" target:self handle:nil];
    }
}

#pragma mark - WKNavigationDelegate
//开始加载
- (void)webView:(WKWebView *)webView didStartProvisionalNavigation:(WKNavigation *)navigation {
   // NSLog(@"开始加载网页");
    //开始加载网页时展示出progressView
    self.progressView.hidden = NO;
    self.progressView.transform = CGAffineTransformMakeScale(1.0f, 1.5f);
    [self.view bringSubviewToFront:self.progressView];
}

//加载完成
- (void)webView:(WKWebView *)webView didFinishNavigation:(WKNavigation *)navigation {
   // NSLog(@"加载完成");
    self.progressView.hidden = YES;
}

//加载失败
- (void)webView:(WKWebView *)webView didFailProvisionalNavigation:(WKNavigation *)navigation withError:(NSError *)error {
    //NSLog(@"加载失败");
    self.progressView.hidden = YES;
}

- (void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler {
    NSLog(@"---------->");
    NSLog(@"%@", navigationAction.request.URL.absoluteString);
    BOOL isAllow = [self handleRequest:navigationAction.request];

    if (isAllow) {
        decisionHandler(WKNavigationActionPolicyAllow);
    } else {
        decisionHandler(WKNavigationActionPolicyCancel);
    }
}

- (void)webView:(WKWebView *)webView didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge completionHandler:(void (^)(NSURLSessionAuthChallengeDisposition disposition, NSURLCredential * _Nullable credential))completionHandler {
    // 判断服务器采用的验证方法
    if (challenge.protectionSpace.authenticationMethod == NSURLAuthenticationMethodServerTrust) {
        // 如果没有错误的情况下 创建一个凭证，并使用证书
        if (challenge.previousFailureCount == 0) {
            NSURLCredential *credential = [NSURLCredential credentialForTrust:challenge.protectionSpace.serverTrust];
            completionHandler(NSURLSessionAuthChallengeUseCredential, credential);
        } else {
            // 验证失败，取消本次验证
            completionHandler(NSURLSessionAuthChallengeCancelAuthenticationChallenge, nil);
        }
    }else {
        completionHandler(NSURLSessionAuthChallengeCancelAuthenticationChallenge, nil);
        
    }
}

#pragma mark - lazyLoading
//初始化配置
- (WKWebViewConfiguration *)configuration{
    if (!_configuration) {
        _configuration = [[WKWebViewConfiguration alloc] init];
        WKPreferences *preferences = [WKPreferences new];
        preferences.javaScriptCanOpenWindowsAutomatically = YES;
        _configuration.preferences = preferences;
        _configuration.userContentController = self.userContentController;
        _configuration.allowsInlineMediaPlayback = YES;
        if (@available(iOS 13.0, *)) {
            _configuration.defaultWebpagePreferences.preferredContentMode = WKContentModeMobile;
        }
    }
    return _configuration;
}

- (WKUserContentController *)userContentController{
    if (!_userContentController) {
        _userContentController = [[WKUserContentController alloc] init];
        //交互关键代码
       // [_userContentController addScriptMessageHandler:self name:@"webViewApp"];
    }
    return _userContentController;
}

- (UIProgressView *)progressView{
    if (!_progressView) {
        _progressView = [[UIProgressView alloc]initWithFrame:CGRectMake(0, getRectNavAndStatusHight, [[UIScreen mainScreen] bounds].size.width, 2)];
        _progressView.backgroundColor = [UIColor blueColor];
        _progressView.transform = CGAffineTransformMakeScale(1.0f, 1.5f);
        [self.view addSubview:_progressView];
    }
    return _progressView;
}

#pragma mark - dealloc
- (void)dealloc {
    
}
@end
